@@ -25,8 +25,6 @@
 #endif
 #include "xf86.h"
 #include "xf86_OSproc.h"
-#include "xf86Resources.h"
-#include "xf86RAC.h"
 #include "xf86cmap.h"
 #include "compiler.h"
 #include "mibstore.h"
@@ -154,6 +152,12 @@ static void ASTSetClippingRectangle(ScrnInfoPtr pScrn,
 static void ASTDisableClipping(ScrnInfoPtr pScrn); 
 static void ASTSetHWClipping(ScrnInfoPtr pScrn, int delta_y);
 
+static void AIPSubsequentSolidTwoPointLine(ScrnInfoPtr pScrn,
+                                           int x1, int y1, int x2, int y2, int flags);                                       
+static void AIPSubsequentDashedTwoPointLine(ScrnInfoPtr pScrn,
+                                            int x1, int y1, int x2, int y2,
+                                            int flags, int phase);
+
 Bool
 ASTAccelInit(ScreenPtr pScreen)
 {
@@ -191,17 +195,33 @@ ASTAccelInit(ScreenPtr pScreen)
     /* Solid Lines */
     if (pAST->ENGCaps & ENG_CAP_SolidLine)
     {    
-        infoPtr->SetupForSolidLine = ASTSetupForSolidLine;
+        if ( (pAST->jChipType == AST2300) || (pAST->jChipType == AST1180) )
+    	{
+            infoPtr->SubsequentSolidTwoPointLine = AIPSubsequentSolidTwoPointLine;
+        }
+        else
+    	{
+            infoPtr->SubsequentSolidTwoPointLine = ASTSubsequentSolidTwoPointLine;
+        }
+
+        infoPtr->SetupForSolidLine = ASTSetupForSolidLine;            
         infoPtr->SubsequentSolidHorVertLine = ASTSubsequentSolidHorVertLine;    
-        infoPtr->SubsequentSolidTwoPointLine = ASTSubsequentSolidTwoPointLine;
         infoPtr->SolidLineFlags = NO_PLANEMASK;
     }
 
     /* Dashed Lines */
     if (pAST->ENGCaps & ENG_CAP_DashedLine)
-    {        
-        infoPtr->SetupForDashedLine = ASTSetupForDashedLine;
-        infoPtr->SubsequentDashedTwoPointLine = ASTSubsequentDashedTwoPointLine;
+    {
+        if ( (pAST->jChipType == AST2300) || (pAST->jChipType == AST1180) )
+        {
+            infoPtr->SubsequentDashedTwoPointLine = AIPSubsequentDashedTwoPointLine;
+        }
+        else
+        {
+            infoPtr->SubsequentDashedTwoPointLine = ASTSubsequentDashedTwoPointLine;
+        }        
+
+        infoPtr->SetupForDashedLine = ASTSetupForDashedLine;        
         infoPtr->DashPatternMaxLength = 64;
         infoPtr->DashedLineFlags = NO_PLANEMASK |
 			           LINE_PATTERN_MSBFIRST_LSBJUSTIFIED;
@@ -1107,7 +1127,7 @@ ASTSetupForColor8x8PatternFill(ScrnInfoPtr pScrn, int patx, int paty,
     pAST->ulCMDReg = cmdreg;
     cpp = (pScrn->bitsPerPixel + 1) / 8;
     pataddr = (CARD32 *)(pAST->FBVirtualAddr +
-                        (paty * pAST->VideoModeInfo.ScreenWidth) + (patx * cpp));   
+                        (paty * pAST->VideoModeInfo.ScreenPitch) + (patx * cpp));   
     ulPatSize = 8*8*cpp;
                 
     if (!pAST->MMIO2D)                    
@@ -1528,4 +1548,306 @@ ASTDisableClipping(ScrnInfoPtr pScrn)
     pAST->EnableClip = FALSE;
 }
 
+static void AIPSubsequentSolidTwoPointLine(ScrnInfoPtr pScrn,
+                                           int x1, int y1, int x2, int y2, int flags)
+{
+ 
+    ASTRecPtr 	pAST = ASTPTR(pScrn);
+    PKT_SC 	*pSingleCMD;
+    ULONG 	dstbase, ulCommand;
+    ULONG	miny, maxy;         
+/*
+    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ASTSubsequentSolidTwoPointLine\n");
+*/    
+
+    /* Modify Reg. Value */
+    ulCommand = (pAST->ulCMDReg & (~CMD_MASK)) | CMD_LINEDRAW | CMD_NORMAL_LINE;
+    if(flags & OMIT_LAST)
+        ulCommand |= CMD_NOT_DRAW_LAST_PIXEL;
+    else
+        ulCommand &= ~CMD_NOT_DRAW_LAST_PIXEL;
+    if (pAST->EnableClip)
+        ulCommand |= CMD_ENABLE_CLIP;
+    else
+        ulCommand &= ~CMD_ENABLE_CLIP;        
+    dstbase = 0;
+    miny = (y1 > y2) ? y2 : y1;
+    maxy = (y1 > y2) ? y1 : y2;
+    if(maxy >= pScrn->virtualY) {
+        dstbase = pAST->VideoModeInfo.ScreenPitch * miny;
+        y1 -= miny;
+        y2 -= miny;
+    }
+
+    if (!pAST->MMIO2D)                    
+    {                  
+        /* Write to CMDQ */    
+        pSingleCMD = (PKT_SC *) pjRequestCMDQ(pAST, PKT_SINGLE_LENGTH*5);
+
+        ASTSetupDSTBase(pSingleCMD, dstbase);
+        pSingleCMD++;    
+        AIPSetupLineXY(pSingleCMD, x1, y1);
+        pSingleCMD++;
+        AIPSetupLineXY2(pSingleCMD, x2, y2);
+        pSingleCMD++;
+        AIPSetupLineNumber(pSingleCMD, 0);
+        pSingleCMD++;                     
+        ASTSetupCMDReg(pSingleCMD, ulCommand);        
+              
+        /* Update Write Pointer */
+        mUpdateWritePointer;                
+
+        /* Patch KDE pass abnormal point, ycchen@052507 */
+        vWaitEngIdle(pScrn, pAST);
+        
+    }
+    else
+    {                  
+        ASTSetupDSTBase_MMIO(dstbase);
+        AIPSetupLineXY_MMIO(x1, y1);
+        AIPSetupLineXY2_MMIO(x2, y2);
+        AIPSetupLineNumber_MMIO(0);
+        ASTSetupCMDReg_MMIO(ulCommand);        
+             
+        vWaitEngIdle(pScrn, pAST);
+      
+    }
+
+                
+} /* end of AIPSubsequentSolidTwoPointLine */
+
+static void
+AIPSubsequentDashedTwoPointLine(ScrnInfoPtr pScrn,
+                                int x1, int y1, int x2, int y2,
+                                int flags, int phase)
+{
+ 
+    ASTRecPtr 	pAST = ASTPTR(pScrn);
+    PKT_SC 	*pSingleCMD;
+    ULONG 	dstbase, ulCommand; 
+    ULONG	miny, maxy;  
+/*
+    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "ASTSubsequentDashedTwoPointLine\n");
+*/   
+
+    /* Modify Reg. Value */
+    ulCommand = pAST->ulCMDReg | CMD_NORMAL_LINE;
+    if(flags & OMIT_LAST)
+        ulCommand |= CMD_NOT_DRAW_LAST_PIXEL;
+    else
+        ulCommand &= ~CMD_NOT_DRAW_LAST_PIXEL;
+    if (pAST->EnableClip)
+        ulCommand |= CMD_ENABLE_CLIP;
+    else
+        ulCommand &= ~CMD_ENABLE_CLIP;        
+    dstbase = 0;        
+    miny = (y1 > y2) ? y2 : y1;
+    maxy = (y1 > y2) ? y1 : y2;
+    if(maxy >= pScrn->virtualY) {
+        dstbase = pAST->VideoModeInfo.ScreenPitch * miny;
+        y1 -= miny;
+        y2 -= miny;
+    }
+
+    if (!pAST->MMIO2D)                    
+    {                  
+        /* Write to CMDQ */    
+        pSingleCMD = (PKT_SC *) pjRequestCMDQ(pAST, PKT_SINGLE_LENGTH*5);
+
+        ASTSetupDSTBase(pSingleCMD, dstbase);
+        pSingleCMD++;    
+        AIPSetupLineXY(pSingleCMD, x1, y1);
+        pSingleCMD++;
+        AIPSetupLineXY2(pSingleCMD, x2, y2);
+        pSingleCMD++;
+        AIPSetupLineNumber(pSingleCMD, 0);
+        pSingleCMD++;                     
+        ASTSetupCMDReg(pSingleCMD, ulCommand);        
+              
+        /* Update Write Pointer */
+        mUpdateWritePointer;
+
+        /* Patch KDE pass abnormal point, ycchen@052507 */
+        vWaitEngIdle(pScrn, pAST);
+              
+    }
+    else
+    {                  
+        ASTSetupDSTBase_MMIO(dstbase);
+        AIPSetupLineXY_MMIO(x1, y1);
+        AIPSetupLineXY2_MMIO(x2, y2);
+        AIPSetupLineNumber_MMIO(0);
+        ASTSetupCMDReg_MMIO(ulCommand);
+       
+        vWaitEngIdle(pScrn, pAST);
+      
+    }
+                	
+}
+
+#ifdef	AstVideo
+/*
+ * Video Part
+ * by ic_yang
+ */
+#include "fourcc.h"
+
+void ASTDisplayVideo(ScrnInfoPtr pScrn, ASTPortPrivPtr pPriv, RegionPtr clipBoxes, int id)
+{
+    ASTPtr              pAST = ASTPTR(pScrn);
+    int                 nBoxs; 
+    int                 ScaleFactorH, ScaleFactorV;
+    ULONG               InitScaleFactorH, InitScaleFactorV;
+    BURSTSCALECMD       CopyCmd = {0};
+    PBURSTSCALECMD      pCopyCmd = NULL;
+    float               fScaleX = 0, fScaleY = 0;
+    xRectangle          rect;
+    BoxPtr              pBox = NULL;
+    short               lSrcX, lSrcY;
+    ULONG               dwCmd = 0;
+    int                 i;
+    
+    pBox = REGION_RECTS(clipBoxes);
+    nBoxs = REGION_NUM_RECTS(clipBoxes);
+
+    xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "MMIO=%x,pBox=%x, nBoxs=%x\n", pAST->MMIO2D, pBox, nBoxs);
+    
+    if(0==pPriv->drw_w || 0==pPriv->drw_h)
+    {
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "drwx=%x, drwy=%x\n", pPriv->drw_w, pPriv->drw_h);
+        return;
+    }
+
+    /* calc scaling factor */
+    fScaleX = (float)pPriv->src_w /(float)pPriv->drw_w;
+    fScaleY = (float)pPriv->src_h /(float)pPriv->drw_h;
+    
+    if (pPriv->src_w == pPriv->drw_w)
+        ScaleFactorH = 0x8000;
+    else
+        ScaleFactorH = (ULONG)((pPriv->src_w-1)*0x8000)/pPriv->drw_w;
+
+    if (pPriv->src_h == pPriv->drw_h)
+    {
+        ScaleFactorV = 0x8000;
+		dwCmd |= SCALE_EQUAL_VER; /* Setting it save the bandwidtch */
+   	}
+    else
+   	{
+        ScaleFactorV = (ULONG)((pPriv->src_h-1)*0x8000)/pPriv->drw_h;
+    }
+
+    if (pPriv->drw_w >= pPriv->src_w)
+        InitScaleFactorH = 0;
+    else
+        InitScaleFactorH = 0x4000;
+
+    if (pPriv->drw_h >= pPriv->src_h)
+        InitScaleFactorV = 0;
+    else
+        InitScaleFactorV = 0x4000;
+
+    switch(pScrn->bitsPerPixel) 
+    {
+    case 32:
+        dwCmd   = CMD_COLOR_32;
+        break;
+    case 16:
+        dwCmd   = CMD_COLOR_16;
+        break;
+    case  8:
+        dwCmd   = CMD_COLOR_08;
+        break;
+    }
+
+    dwCmd |= CMD_TYPE_SCALE;
+    if (pPriv->drw_w >= pPriv->src_w)
+        dwCmd |= SCALE_SEG_NUM_1;
+    else
+        dwCmd |= SCALE_SEG_NUM_2;
+
+    dwCmd |= SCALE_FORMAT_YUV2RGB;
+    switch(id)
+    {
+    case PIXEL_FMT_YUY2:
+        dwCmd |= YUV_FORMAT_YUYV; 
+        break;
+    case PIXEL_FMT_UYVY:        
+        dwCmd |= YUV_FORMAT_UYVY; 
+        break;
+    default:
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Other pix format:%x\n", id);
+        break;
+    }
+    
+    for(i = 0; i < nBoxs; i++, pBox++)
+    {
+        rect.x = pBox->x1 - pPriv->drw_x;
+        rect.y = pBox->y1 - pPriv->drw_y;
+        rect.width = pBox->x2 - pBox->x1;
+        rect.height = pBox->y2 - pBox->y1;        
+        
+        lSrcX = (ULONG)((float)rect.x * fScaleX + pPriv->src_x + 0.5f);
+        lSrcY = (ULONG)((float)rect.y * fScaleY + pPriv->src_y + 0.5f);
+              
+        pCopyCmd = (BURSTSCALECMD*)pjRequestCMDQ(pAST, PKT_TYPESCALE_LENGTH);  
+        
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "pCopyCmd=%p, pBox=%x,%x,%x,%x\n", pCopyCmd, pBox->x1, pBox->y1, pBox->x2, pBox->y2);                
+       
+        CopyCmd.dwHeader0        = (ULONG)  PKT_BURST_CMD_HEADER0 |
+                                             PKT_TYPESCALE_DATALENGTH |
+                                             PKT_TYPESCALE_ADDRSTART  |
+                                             BURST_FORCE_CMD;
+
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "CopyCmd.dwHeader0=%x\n", CopyCmd.dwHeader0);
+
+        CopyCmd.dwSrcPitch       = (ULONG)(pPriv->srcPitch << 16);
+        
+        CopyCmd.dwDstHeightPitch = (ULONG)  ((pAST->VideoModeInfo.ScreenPitch << 16) | 0xFFFF);
+        
+        CopyCmd.dwDstXY          = (ULONG) ((pBox->x1 << 16) | (pBox->y1 & 0xFFFF));                                                 
+        CopyCmd.dwSrcXY          = (ULONG) ((lSrcX << 16) | (lSrcY & 0xFFFF));                                                
+        CopyCmd.dwRecHeightWidth = (ULONG) ((rect.width << 16) | rect.height);                                                 
+                                                     
+        CopyCmd.dwInitScaleFactorH = InitScaleFactorH;
+        CopyCmd.dwInitScaleFactorV = InitScaleFactorV;
+        CopyCmd.dwScaleFactorH   = ScaleFactorH; 
+        CopyCmd.dwScaleFactorV   = ScaleFactorV;
+
+        CopyCmd.dwSrcBaseAddr = pPriv->bufAddr[pPriv->currentBuf];
+        CopyCmd.dwDstBaseAddr = 0;
+        CopyCmd.dwCmd = dwCmd;
+        CopyCmd.NullData[0] = 0;    /* for alignment */
+        memcpy(pCopyCmd, &CopyCmd, sizeof(CopyCmd));
+        
+        mUpdateWritePointer;
+        
+#if 0        
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "%08x, %08x, %08x, %08x\n",
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8000),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8004),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8008),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x800C));
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "%08x, %08x, %08x, %08x\n",
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8010),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8014),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8018),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x801C));
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "%08x, %08x, %08x, %08x\n",
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8020),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8024),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8028),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x802C));
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "%08x, %08x, %08x, %08x\n",
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8030),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8034),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x8038),
+                        *(ULONG *)(pAST->MMIOVirtualAddr+0x803C));
+#endif
+
+    } /* End of for-loop */
+  
+} /* ASTDisplayVideo */
+#endif	/* AstVideo */
+
 #endif	/* end of Accel_2D */
